package edu.berkeley.eecs.openwsn.client;

import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Widget;

import edu.berkeley.eecs.openwsn.server.DomainController;
import edu.berkeley.eecs.openwsn.shared.RequestEvent;
import com.google.gwt.user.client.ui.IntegerBox;
import com.google.gwt.user.cellview.client.CellTree;
import com.google.gwt.view.client.TreeViewModel;
import com.google.gwt.view.client.AbstractDataProvider;
import com.google.gwt.view.client.ListDataProvider;
import com.google.gwt.view.client.SelectionModel.AbstractSelectionModel;
import com.google.gwt.view.client.NoSelectionModel;
import com.google.gwt.view.client.TreeViewModel.NodeInfo;
import com.google.gwt.view.client.TreeViewModel.DefaultNodeInfo;
import com.google.gwt.cell.client.TextCell;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.user.client.ui.TextBox;
/**
 * 
 * @author Xavi Vilajosana
 *
 */
public class MainWindow extends Composite {
	public Long netId = null;
	public String ipAddress = null;

	private static MainWindowUiBinder uiBinder = GWT
			.create(MainWindowUiBinder.class);

	interface MainWindowUiBinder extends UiBinder<Widget, MainWindow> {
	}


	/**
	 * Create a remote service proxy to talk to the server-side Greeting service.
	 */
	private final GreetingServiceAsync greetingService = GWT
			.create(GreetingService.class);
	@UiField ListBox netlistbox;
	@UiField ListBox iplistbox;
	@UiField IntegerBox intervalintegerbox;
	@UiField IntegerBox temperaturemaxintegerbox;
	@UiField IntegerBox temperatureminintegerbox;
	@UiField IntegerBox humiditymaxintegerbox;
	@UiField IntegerBox humidityminintegerbox;
	@UiField TextBox emailtextbox;

	public MainWindow() {
		initWidget(uiBinder.createAndBindUi(this));

		getNetworks();

		greetingService.getTemperatureMaxAlarm(new AsyncCallback<Integer>() {
			@Override
			public void onSuccess(Integer result) {
				temperaturemaxintegerbox.setValue(result);
			}

			@Override
			public void onFailure(Throwable caught) {
				ServerError();
			}
		});

		greetingService.getTemperatureMinAlarm(new AsyncCallback<Integer>() {
			@Override
			public void onSuccess(Integer result) {
				temperatureminintegerbox.setValue(result);
			}

			@Override
			public void onFailure(Throwable caught) {
				ServerError();
			}
		});

		greetingService.getHumidityMaxAlarm(new AsyncCallback<Integer>() {
			@Override
			public void onSuccess(Integer result) {
				humiditymaxintegerbox.setValue(result);
			}

			@Override
			public void onFailure(Throwable caught) {
				ServerError();
			}
		});

		greetingService.getHumidityMinAlarm(new AsyncCallback<Integer>() {
			@Override
			public void onSuccess(Integer result) {
				humidityminintegerbox.setValue(result);
			}

			@Override
			public void onFailure(Throwable caught) {
				ServerError();
			}
		});
		
		greetingService.getInterval(new AsyncCallback<Integer>() {
			@Override
			public void onSuccess(Integer result) { intervalintegerbox.setValue(result); }
			@Override
			public void onFailure(Throwable caught) { ServerError(); }
		});
		
		greetingService.getEmail(new AsyncCallback<String>() {
			@Override
			public void onSuccess(String result) { emailtextbox.setText(result); }
			@Override
			public void onFailure(Throwable caught) { ServerError(); }
		});
	}

	private void getNetworks() {
		netlistbox.clear();
		netlistbox.addItem("--");
		greetingService.getNetworks(new AsyncCallback<List<Long>>() {

			@Override
			public void onSuccess(List<Long> result) {
				for (Long l:result){
					netlistbox.addItem(l.toString());
				}

			}

			@Override
			public void onFailure(Throwable caught) {
				Window.alert("An error ocurred in the server.");
			}
		});
	}

	@UiHandler("netlistbox")
	void onChangeNetListBox(ChangeEvent sender) {
		netId = netlistbox.getSelectedIndex() == 0 ? null 
				: Long.parseLong(netlistbox.getItemText(netlistbox.getSelectedIndex()));

		if (netlistbox.getSelectedIndex()>0){
			String text=netlistbox.getItemText(netlistbox.getSelectedIndex());
			//iplistbox.clear();
			getSensors(text);

		}else if (netlistbox.getSelectedIndex()==0){
			/*iplistbox.clear();
			iplistbox.addItem("--");
			applistbox.clear();
			applistbox.addItem("--");
			getNetworks();//just in case there is an update.*/
		}

	}


	@UiHandler("netlistbox")
	void onClickNetListBox(ClickEvent sender) {
		if (netlistbox.getSelectedIndex()>0){
			//do nothing
		}else if (netlistbox.getSelectedIndex()==0){
			iplistbox.clear();
			iplistbox.addItem("--");
			/*applistbox.clear();
			applistbox.addItem("--");*/
			getNetworks();//just in case there is an update.

			//generate event to all listeners.
			EventBus.getInstance().fireEvent(new RequestEvent(null, null, null,true));
		}

	}


	private void getSensors(String text) {
		greetingService.getSensors(Long.parseLong(text),new AsyncCallback<List<String>>() {

			@Override
			public void onFailure(Throwable caught) {
				Window.alert("An error occurred when searching sensors in the server.");

			}

			@Override
			public void onSuccess(List<String> result) {
				iplistbox.clear();
				iplistbox.addItem("--");
				for (String s:result){
					iplistbox.addItem(s);
				}

			}

		});
	}


	@UiHandler("iplistbox")
	void onChangeIPListBox(ChangeEvent sender) {
		ipAddress = iplistbox.getSelectedIndex() == 0 ? null
				: iplistbox.getItemText(iplistbox.getSelectedIndex());
	}

	public void reset() {
		iplistbox.clear();
		iplistbox.addItem("--");
		//applistbox.clear();
		//applistbox.addItem("--");
		netlistbox.clear();
		netlistbox.addItem("--");
	}

	private void ServerError() {
		Window.alert("An error ocurred in the server");
	}
	@UiHandler("temperaturemaxintegerbox")
	void onTemperaturemaxintegerboxValueChange(ChangeEvent event) {
		setTemperatureAlarm();
	}
	
	@UiHandler("temperatureminintegerbox")
	void onTemperatureminintegerboxValueChange(ChangeEvent event) {
		setTemperatureAlarm();
	}

	private void setTemperatureAlarm() {
		greetingService.setTemperatureAlarm(temperatureminintegerbox.getValue(), 
				temperaturemaxintegerbox.getValue(), new AsyncCallback<Void>() {
			@Override
			public void onSuccess(Void result) { }
			@Override
			public void onFailure(Throwable caught) {
				ServerError();
			}
		});
	}
	@UiHandler("intervalintegerbox")
	void onIntervalintegerboxValueChange(ChangeEvent event) {
		greetingService.setInterval(intervalintegerbox.getValue(), new AsyncCallback<Void>() {
			@Override public void onSuccess(Void result) { }
			@Override public void onFailure(Throwable caught) { ServerError(); }
		});
	}
	@UiHandler("humiditymaxintegerbox")
	void onHumiditymaxintegerboxValueChange(ChangeEvent event) {
		setHumidityAlarm();
	}
	
	@UiHandler("humidityminintegerbox")
	void onHumidityminintegerboxValueChange(ChangeEvent event) {
		setHumidityAlarm();
	}
	
	private void setHumidityAlarm() {
		greetingService.setHumidityAlarm(humidityminintegerbox.getValue(), 
				humiditymaxintegerbox.getValue(), new AsyncCallback<Void>() {
			@Override public void onSuccess(Void result) { }
			@Override public void onFailure(Throwable caught) { ServerError(); }
		});
	}
	@UiHandler("emailtextbox")
	void onEmailtextboxValueChange(ChangeEvent event) {
		greetingService.setEmail(emailtextbox.getText(), new AsyncCallback<Void>() {
			@Override public void onSuccess(Void result) { }
			@Override public void onFailure(Throwable caught) { ServerError(); }
		});
	}
}
