package edu.berkeley.eecs.openwsn.client;

import java.awt.Point;
import java.util.Date;
import java.util.List;

import java_cup.Main;

import org.moxieapps.gwt.highcharts.client.Axis;
import org.moxieapps.gwt.highcharts.client.Chart;
import org.moxieapps.gwt.highcharts.client.Credits;
import org.moxieapps.gwt.highcharts.client.Legend;
import org.moxieapps.gwt.highcharts.client.Series;
import org.moxieapps.gwt.highcharts.client.ToolTip;
import org.moxieapps.gwt.highcharts.client.ToolTipData;
import org.moxieapps.gwt.highcharts.client.ToolTipFormatter;
import org.moxieapps.gwt.highcharts.client.labels.DataLabels;
import org.moxieapps.gwt.highcharts.client.plotOptions.BarPlotOptions;

import sun.awt.VerticalBagLayout;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.i18n.client.NumberFormat;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.RootPanel;

import edu.berkeley.eecs.openwsn.shared.DataRequestEventHandler;
import edu.berkeley.eecs.openwsn.shared.RequestEvent;
import edu.berkeley.eecs.openwsn.shared.SensorStats.Types;
import edu.berkeley.eecs.openwsn.shared.TupleDateValue;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.LayoutPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * @author Xavi Vilajosana
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class OpenWSNCharts implements DataRequestEventHandler, EntryPoint  {
	/**
	 * The message displayed to the user when the server cannot be reached or
	 * returns an error.
	 */
	private static final String SERVER_ERROR = "An error occurred while "
			+ "attempting to contact the server. Please check your network "
			+ "connection and try again.";

	/**
	 * Create a remote service proxy to talk to the server-side Greeting service.
	 */
	private final GreetingServiceAsync greetingService = GWT
			.create(GreetingService.class);

	private Date lastseen=new Date(0);//1970
	
	/*private String ipAddress=null;
	private Long netId=null;*/

	private Chart temperatureChart=null;
	private Chart humidityChart=null;
	private Series temperatureSeries=null;
	private Series humiditySeries=null;
	private MainWindow main=null;
	
	public Chart createChart(final String appIdString) { 
		Series series = null;
		final Chart chart = new Chart()  
		.setType(Series.Type.SPLINE)  
		.setMarginRight(10)  
		.setChartTitleText("Live data")  
		.setBarPlotOptions(new BarPlotOptions()  
		.setDataLabels(new DataLabels()  
		.setEnabled(true)  
				)  
				)  
				.setLegend(new Legend()  
				.setEnabled(false)  
						)  
						.setCredits(new Credits()  
						.setEnabled(false)  
								)  
								.setToolTip(new ToolTip()  
								.setFormatter(new ToolTipFormatter() {  
									public String format(ToolTipData toolTipData) {  
										return "<b>" + toolTipData.getSeriesName() + "</b><br/>" +  
												DateTimeFormat.getFormat("yyyy-MM-dd HH:mm:ss")  
												.format(new Date(toolTipData.getXAsLong())) + "<br/>" +  
												NumberFormat.getFormat("0.00").format(toolTipData.getYAsDouble());  
									}  
								})  
										);  

		chart.getXAxis()  
		.setType(Axis.Type.DATE_TIME)  
		.setTickPixelInterval(150);  

		chart.getYAxis()  
		.setAxisTitleText("Value")  
		.setPlotLines(  
				chart.getYAxis().createPlotLine()  
				.setValue(0)  
				.setWidth(1)  
				.setColor("#808080")  
				);  

		if(appIdString == "Temperature")
			series = temperatureSeries = chart.createSeries();
		else 
			series = humiditySeries = chart.createSeries();
		chart.addSeries(series  
				.setName("Data")  
				);  



		return chart;  
	} 

	/**
	 * This is the entry point method.
	 */
	public void onModuleLoad() {
		EventBus.getInstance().addHandler(RequestEvent.TYPE, this);
		
		RootPanel rootPanel = RootPanel.get("chart");
		
		VerticalPanel chartPanel = new VerticalPanel();
		
		temperatureChart=createChart("Temperature");
		temperatureChart.setChartTitleText("Temperature");
		temperatureChart.getYAxis().setAxisTitleText("Degrees Celsius (℃)");
		chartPanel.add(temperatureChart);
		
		humidityChart = createChart("Humidity");
		humidityChart.setChartTitleText("Humidity");
		humidityChart.getYAxis().setAxisTitleText("Percent (%)");
		chartPanel.add(humidityChart);
		
		chartPanel.setWidth("100%");
		rootPanel.add(chartPanel);
				
		RootPanel controlPanel=RootPanel.get("control");
		main=new MainWindow();
		controlPanel.add(main);


		Timer timer = new Timer() {  
			@Override  
			public void run() { 
				if (main.netId != null && main.ipAddress != null){
					getDataFromServer(temperatureChart, temperatureSeries, "Temperature");
					getDataFromServer(humidityChart, humiditySeries, "Humidity");
				}
			}

			private void getDataFromServer(final Chart chart,
					final Series series, final String appIdString) {
				greetingService.getNewValues(main.netId, main.ipAddress,appIdString, lastseen, new AsyncCallback<List<TupleDateValue>>() {
					public void onFailure(Throwable caught) {
						// Show the RPC error message to the user
						Window.alert("Error");
					}

					public void onSuccess(List<TupleDateValue> result) {
						for (TupleDateValue tup:result){ 
							if (series.getPoints().length>40){
								Date d=tup.getDate();
								Double val=tup.getValue();
								series.addPoint( d.getTime(),  val,false, true, true );
							}else{
								Date d=tup.getDate();
								Double val=tup.getValue();
								series.addPoint( d.getTime(),  val,false, false, true );
							}
							lastseen=tup.getDate();
						}
						chart.redraw();
					}
				});
			}  
		};  
		timer.scheduleRepeating(2000); 		
	}

	@Override
	public void requestData(RequestEvent event) {
		//	invoked by the eventbus producers.
		//clear the chart.
		temperatureChart.removeAllSeries();
		temperatureSeries=temperatureChart.createSeries();  
		temperatureChart.addSeries(temperatureSeries  
				.setName("Data")  
		);
		temperatureChart.redraw();
		
		humidityChart.removeAllSeries();
		humiditySeries = humidityChart.createSeries();  
		humidityChart.addSeries(humiditySeries  
				.setName("Data")  
		);
		humidityChart.redraw();
		lastseen=new Date(0);
	}
}