/**
 * @file config.h
 * @brief Archivo de configuración con diferentes macros para simplicar el código.
 */
#ifndef CONFIG_H_
#define CONFIG_H_

#include <uartio.h>

/// Nivel actual de depuración.
#define LOG_LEVEL 0

/// Prioridad de la tarea ::TASK_ReadSettings
#define	CONFIGURATION_TASK_PRIORITY			( tskIDLE_PRIORITY + 3)

/// Prioridad de la tarea ::TASK_Sensor
#define	PRODUCER_TASK_PRIORITY				( tskIDLE_PRIORITY + 2)

/// Prioridad de la tarea ::TASK_Sender
#define CONSUMER_TASK_PRIORITY				( tskIDLE_PRIORITY + 1)

/// Tamaño máximo de la cola que comunica ::TASK_Sensor y ::TASK_Sender
#define QUEUE_LENGTH						( 10 )

/// Máscara que selecciona el led de medición.
#define LEDID_MEASURING_MASK				(1 << 9)

/// Máscara que selecciona el led de alarma de temperatura.
#define LEDID_ALARM_TEMPERATURE_MASK		(1 << 8)

/// Máscara que selecciona el led de alarma de humedad.
#define LEDID_ALARM_HUMIDITY_MASK			(1 << 7)

/// Máscara que selecciona el led de microcontrolador ocupado.
#define LEDID_BUSY_MASK						(1 << 6)

/// Identificador del punto de acceso.
#define WLAN_SSID	 						"ONO8F26"

/// Password del punto de acceso.
#define WLAN_PASSWORD 						"WHEFRXSYYYGSFGSRWXYE"

/// Tipo de seguridad del punto de acceso.
#define WLAN_SECURITY_TYPE 					WIFLY_ST_WPA2

/// Evita el warning de parámetro no usado.
#define UNUSED(x) (void)(x)

/// Puerto UART donde se enviarán los logs.
#define UARTID_DEBUG		3

/// Muestra un mensaje de log con el nombre de función y línea.
#define __LOG(FORMAT, ...) UARTIO_Printfln(UARTID_DEBUG, NULL, "%s(%d) %s: " \
		FORMAT, __FILE__, __LINE__, __FUNCTION__, ##__VA_ARGS__)

/// Log de nivel 0.
#define LOG(FORMAT, ...) __LOG(FORMAT, ##__VA_ARGS__)

/// Log de nivel 0.
#define LOG0(FORMAT, ...) LOG(FORMAT, ##__VA_ARGS__)

/// Log de nivel 1.
#define LOG1(FORMAT, ...)

/// Log de nivel 2.
#define LOG2(FORMAT, ...)
#if defined LOG_LEVEL && LOG_LEVEL == 1
	#undef LOG1
	#define LOG1(FORMAT, ...) __LOG(FORMAT, ##__VA_ARGS__)
#endif
#if defined LOG_LEVEL && LOG_LEVEL == 2
	#define LOG2(FORMAT, ...) __LOG(FORMAT, ##__VA_ARGS__)
#endif
#endif /* CONFIG_H_ */
