/**
 * @file server.h
 * @brief Cabecera del driver SERVER.
 */
#ifndef SERVER_H_
#define SERVER_H_

#include <stdint.h>
#include <stdbool.h>

#include <wifly.h>

/// Formato en el que se enviarán los datos al servidor.
#define SERVER_DATAFORMAT "%.1f"

/// Guarda la configuración actual recibida desde el servidor.
typedef struct _SERVER_Settings {
	/// Indica si ha sido inicializado correctamente
	volatile bool initialized;
	/// Tiempo de refresco (en minutos)
	uint32_t interval;
	/// Valor mínimo de temperatura
	int8_t temperatureMin;
	/// Valor máximo de temperatura
	int8_t temperatureMax;
	/// Valor mínimo de humedad
	int8_t humidityMin;
	/// Valor máximo de humedad
	int8_t humidityMax;
} SERVER_Settings;

void SERVER_Initialize(uint32_t wiflyUartId, const char *_ssid,
		WIFLY_SecurityType _securityType, const char *_password);
bool SERVER_SendStats(const char *application, const char *value);
void SERVER_GetSettings(SERVER_Settings *settings);

#endif /* SERVER_H_ */
