/**
 * @file sleep.c
 * @brief Implementación del driver SLEEP.
 */
#include <FreeRTOS.h>

#include <type.h>
#include <task.h>
#include <uartio.h>
#include <portmacro.h>

/**
  * Realiza una espera pasiva. Tiene que llamarse desde dentro de una tarea.
  * \param miliseconds Milisegundos de espera.
  */
inline void SLEEP_PasiveMSSleep(int miliseconds)
{
	const int delay = miliseconds / portTICK_RATE_MS;
	vTaskDelay(delay);
}

/**
 * Realiza una espera activa. Puede llamarse desde fuera de una tarea.
 * \param miliseconds milisegundos de espera.
 */
inline void SLEEP_ActiveMSSleep(int miliseconds)
{ int i; for(i = 0; i < (uint32_t)7140 * miliseconds; i++); }

/**
 * Realiza una espera activa. A diferencia de la anterior esta la realizará
 * en nanosegundos.
 * \param nanoseconds nanosegundos de espera.
 */
inline void SLEEP_ActiveNSSleep(int nanoseconds)
{
	uint32_t i;
	for(i=0; i<( ((uint32_t)7)*nanoseconds); i++){ ; }
}

