/**
 * @file common.h
 * @brief Este archivo contiene la información común del programa. Es compartido por
 * los archivos de código del proyecto.
 */
#ifndef COMMON_H_
#define COMMON_H_

#include <FreeRTOS.h>
#include <LPC17xx.h>

#include <queue.h>

#include <server.h>

/**
 * Tipos de dato que soporta la cola ::xQueue.
 */
typedef enum _QueueDataType
{
	/// Indica que el elemento es de tipo Temperatura.
	QueueDataType_Temperature = 0,
	/// Indica que el elemento es de tipo Humedad.
	QueueDataType_Humidity
} QueueDataType;

/**
 * Estructura que se introducirá por ::TASK_Sensor dentro de la cola ::xQueue y
 * será consumida por la tarea ::TASK_Sender para su envío al servidor.
 */
typedef struct _QueueData
{
	/// Tipo de dato que contiene el elemento.
	QueueDataType type;
	/// Valor de temperatura o humedad.
	float value;
} QueueData;

extern xQueueHandle xQueue;
extern SERVER_Settings settings;

#endif /* COMMON_H_ */
