/**
 * @file hooks.c
 * @brief Contiene los hooks necesarios para el planificador de tareas.
 */
#include "common.h"

#include <task.h>
#include <stdbool.h>

#include <config.h>
#include <led.h>

/**
 * Es llamado por el planificador de tareas cuando tenemos el microcontrolador
 * en modo idle, esperando a continuar la ejecución de alguna de las tareas.
 *
 * Dentro de esta función se activa el modo sleep para tratar de reducir el
 * consumo del microcontrolador.
 *
 * Además cuando se encuentra dentro de esta función se apaga el led
 * ::LEDID_BUSY_MASK que indica que el microcontrolador está ocupado,
 * para dar un feedback visual. Antes de salir de la función se volverá a
 * encender el led ::LEDID_BUSY_MASK.
 */
void vApplicationIdleHook()
{
	LED_Switch(LEDID_BUSY_MASK, false);
	LPC_SC->PCON = 0x0;
	SCB->SCR = 0x0;
	__WFI();
	//SystemCoreClockUpdate();
	LED_Switch(LEDID_BUSY_MASK, true);
}

/**
 * Esta función es llamada por el planificador de tareas cuando se intenta crear
 * una variable usando memoría dinámica y este falla.
 */
void vApplicationMallocFailedHook()
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */

	for( ;; );
}

/**
 * Se llama a esta función cuando ocurre un desbordamiento de pila en alguna de
 * las tareas que están corriendo en el microcontrolador.
 */
void vApplicationStackOverflowHook(xTaskHandle *pxTask,
		signed char *pcTaskName)
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	UNUSED(pxTask);
	LOG("%s", pcTaskName);
	for( ;; );
}

/**
 * Hook donde podríamos implementar una funcionalidad de timer. Para este
 * proyecto no es necesario su uso.
 */
void vApplicationTickHook()
{
	/* This example does not use the tick hook to perform any processing. */
}
