/**
 * @file main.c
 * @brief Contiene la función main y declara las variables globales.
 */
#include "common.h"

#include <task.h>
#include <queue.h>
#include <semphr.h>

#include <config.h>

#include <led.h>

/// Guarda el contenido de la configuración obtenida de Internet.
SERVER_Settings settings = { false, 1 };

// Prototipo de las funciones de las tareas definidas en otros archivos.
extern void TASK_Sender(void *pvParameters);
extern void TASK_Sensor(void *pvParameters);
extern void TASK_ReadSettings(void *pvParameters);

/**
 * Handle de la cola usada para comunicar la tarea ::TASK_Sender
 * con ::TASK_Sensor.
 */
xQueueHandle xQueue = NULL;

/**
  * Punto de inicio de la aplicación.
  * Creará los tasks que necesitamos para realizar el envio de datos. Además
  * inicializa los leds que usaremos y el puerto UART que usaremos para
  * leer los logs a través de un puerto USB con el ordenador.
  */
int main(void)
{
	uint32_t i;
	uint32_t ledMask = LEDID_MEASURING_MASK | LEDID_ALARM_TEMPERATURE_MASK |
			LEDID_ALARM_HUMIDITY_MASK | LEDID_BUSY_MASK;
	UARTIO_Init(UARTID_DEBUG, 115200);
	LOG("Starting...");

	LED_Initialize(ledMask);
	LED_Switch(ledMask, true);
	SLEEP_ActiveMSSleep(2000);
	LED_Switch(ledMask, false);

	/** Crea la cola. */
	xQueue = xQueueCreate(QUEUE_LENGTH, sizeof(QueueData));

	if(NULL != xQueue)
	{
		xTaskCreate(TASK_Sender,
				(signed char *)"Sender Task",
				500,
				NULL,
				CONSUMER_TASK_PRIORITY,
				NULL);
		xTaskCreate(TASK_Sensor,
				(signed char *)"Sensor Task",
				350,
				NULL,
				PRODUCER_TASK_PRIORITY,
				NULL);
		xTaskCreate(TASK_ReadSettings,
				(signed char*)"Read current Configuration Task",
				500,
				NULL,
				CONFIGURATION_TASK_PRIORITY,
				NULL);

		/* Inicia las tareas */
		vTaskStartScheduler();
	}

	for( ;; );
}
