/**
 * @file taskreadsettings.c
 * @brief Contiene la implementación de la tarea ::TASK_ReadSettings.
 */
#include "common.h"
#include <task.h>
#include <config.h>

/**
 * Tarea que se encarga de la lectura de la configuración del servidor. Se
 * obtienen los datos relacionados con las alarmas (mínimos y máximos), y el
 * intervalo de lectura de datos.
 *
 * Esto es guardado en una variable global ::settings.
 *
 * Tras obtener la configuración desde el servidor se queda a la espera tantos
 * minutos como sea el intervalo de lectura.
 *
 * Esta tarea además se encarga de inicializar el dispositivo WiFly ya que es
 * la primera que lo usará.
 * \param pvParameters Contiene los parámetros para pasar a la tarea.
 * \sa ::TASK_Sender
 */
void TASK_ReadSettings(void *pvParameters)
{
	unsigned long stackHWM;
	xTaskHandle taskHandle = xTaskGetCurrentTaskHandle();

	UNUSED(pvParameters);

	LOG("Starting task %s", __FUNCTION__);

	SERVER_Initialize(0, WLAN_SSID, WLAN_SECURITY_TYPE, WLAN_PASSWORD);
	for(;;)
	{
		int32_t value;

		SERVER_GetSettings(&settings);
		xQueueSend(xQueue, &value, 0);
		stackHWM = uxTaskGetStackHighWaterMark(xTaskGetCurrentTaskHandle());
		LOG("Free stack: %d", stackHWM);

		SLEEP_PasiveMSSleep(settings.interval * 60000);
	}
}
