/**
 * @file tasksender.c
 * @brief Contiene la tarea ::TASK_Sender.
 */
#include <FreeRTOS.h>
#include <LPC17xx.h>
#include <task.h>
#include <queue.h>
#include "common.h"
#include <config.h>
#include <server.h>

/**
  * Tarea que consumirá los datos de temperatura o humedad para enviarlos al
  * servidor.
  *
  * Se queda a la espera de recibir nuevos datos y en caso de pasar un tiempo
  * sin recibir nuevos datos ordenará al dispositivo WiFly que se pase al modo
  * sleep usando la función ::WIFLY_Sleep.
  * \param pvParameters Parámetros pasados a la tarea.
  */
void TASK_Sender(void *pvParameters)
{
	unsigned long stackHWM;
	xTaskHandle taskHandle = xTaskGetCurrentTaskHandle();

	UNUSED(pvParameters);

	LOG("Starting task %s", __FUNCTION__);

	for(;;)
	{
		QueueData queueData;
		char *application;
		char valueString[8];

		if(xQueueReceive(xQueue, &queueData, 5000 / portTICK_RATE_MS))
		{
			uint32_t retries = 0;
			switch(queueData.type)
			{
			case QueueDataType_Temperature:
				LOG0("Temperature received");
				application = "Temperature"; break;
			case QueueDataType_Humidity:
				LOG0("Humidity received");
				application = "Humidity"; break;
			default:
				continue;
			}
			sprintf(valueString, SERVER_DATAFORMAT, queueData.value);
			LOG1("Value received: %s for %s... Sending it to server",
					valueString, application);
			while(false == SERVER_SendStats(application, valueString))
				LOG1("Retry: %d", ++retries);
			stackHWM = uxTaskGetStackHighWaterMark(xTaskGetCurrentTaskHandle());
			LOG("Free stack: %d", stackHWM);
		}
		else
		{
			WIFLY_Sleep();
			SLEEP_PasiveMSSleep(30000);
		}
	}
}
